# Thesis-Project

Numerical experiments on spin control.

Yuning Zhang, TU Delft, MEP

## Workflow

### Solid Development in SpinControl.jl

1. Architecture/Interface Design
2. Implementation/Debugging
3. UnitTests/Precompile

### Numerical Experiments

1. Initial Exploration (Jupyter Notebook)
2. Data Generation (Julia Scripts)
   1. Parameter Settings (`Simulation`)
   2. Revoke Simulation (Methods of `Simulation`)
   3. Data Export: meaningful names matters
   4. Data Storage

### Visualization and Presentation

1. Data Visualization (Jupyter Notebook+Scripts)
   1. Data Loading
   2. Basic Exploration
   3. Iterative Visualization: publication standards
   4. Fixed Production (Reorganize into Scripts)
   5. Figures Export: meaningful names matters
2. Experimental Report (Jupyter?)
   1. Figure Rescale
   2. Full Details

## Best Practice

### Clear Organization

Productivity comes from a clear organization of things.  

- The architecture of code.
- The directory of files.
- An efficient workflow that links everything together.

A good structure is the key of order and order boosts productivity, while chaos lead to mess. You must constantly reorganize your code and files to reduce entropy and maintain order.

The following things are useful to remind

- Seperation of functionalities is necessary. Build serious code in source files. Demonstrate data and plot figures in notebooks.
- Use notebook as a coding playground is acceptable only when code size is small. Once you need complicated logics, make it in scripts.
- Think before doing things. Think about what's the best way to do it and how can you make it more efficient.

Design or at least imagine a architecture before coding. Make meaningful file structures. Look into things and make abstraction at proper level.

### Simulation Automation

- A simulation is defined by a set of necessary parameters, with full details. The abstraction is implemented as `Simulation`.
- A print function should be implemented to document the parameters of the simulation, which can be then exported to text files.
- Specific methods of `Simulation` should be implemented to execute actual simulation, then export the data, and document the meta information.
- Each `Simulation` should have a unique ID, for later reference. `objectid`.
- Deploy simulations in scripts and include all scripts in a single shell file.

### Demonstration

- The correct way to use Jupyter notebook is to use it as an enhanced markdown or LaTeX for teaching, demonstration, reports and notes.
- Invoke code only for visualizations if you write report in Jupyter notebook.
- Take care of all the details like colors, legends, or even ticks in logscale. One bad apple in a barral can destroy everything.
- Make illustrative figures. Use everything to deliver useful information. A good scientific figure should tell a beautiful story.
- Save the figures with meaningful name for later usage.

## Directory

./data/

## Tasks


- [ ] Use JSON format for meta information 
- [ ] Derivation of Sensitivity
- [ ] Unit of Sensitivity
- [X] Paper Reading
- [ ] Dimensionless unit
- [X] Axis and duration error
- [X] Fidelity
- [X] Unitary Sampling
- [ ] Reading Monte-Carlo
- [X] SquarePulse Type
- [X] Correct Logscale Plot.
- [X] Rethink workflow.
  - [X] Implement `Simulation`
  - [X] Seperate date generation/visualization
  - [X] Scripts to generate data
  - [X] Jupyter to visualize data
- [X] Ensemble Rabi Analysis 3D.
  - [X] Envelope Error with Time
  - [X] Phase Error with Time
  - [X] Offset and Unsymmetry
- [X] Analysis 2D
- [X] Analysis 1D
