import os
os.chdir("..")
import pandas as pd
import matplotlib.pyplot as pyplot
import matplotlib as plt
import numpy as np
import pandas as pd
import os 

def find_id_pattern(patterns, data_dir=""):
    for fn in os.scandir(data_dir):
        _, f_ext = os.path.splitext(fn)
        if f_ext==".siminfo":
            with open(fn, "r") as f:
                info=f.read()
                if all([pattern in info for pattern in patterns]):
                    return info.split("\n")[3].strip()[4:]

def load_sim(data_id, data_dir="", verbose=True):
    if verbose:
        with open(data_dir+"simulation_"+data_id+".siminfo", "r") as f:
            print(f.read()) 
    data=pd.read_csv(data_dir+"simulation_"+data_id+".csv")
    return data

def delete_sim(data_id, data_dir="", fmt=".csv"):
    sim_info=data_dir+"simulation_"+data_id+".siminfo"
    sim_data=data_dir+"simulation_"+data_id+fmt

    for fn in (sim_info, sim_data):
        if os.path.exists(fn):
            os.remove(fn)
            if verbose:
                print(fn, "  removed!")

def sci_fmt(n):
    if n==0:
        return "0.0"
    s="{:.1e}".format(n)
    ne,np=s.split("e")
    return f"${ne}\\times 10^{{{int(np)}}}$"


def read_siminfo(data_id, data_dir=""):
    with open(data_dir+"simulation_"+data_id+".siminfo", "r") as f:
        info=f.read()
        items = [s.strip() for s in info.split("\n")][3:-2]
    return _parse_item(items)

def _parse_item(items):
    dic = dict()
    for i in items:
        pair=i.split(":") 
        dic[pair[0]]=pair[1].strip()
    return dic


def plot_driving_error(param_set : tuple, out_dir : str, rotation : bool =False):
    data, h, dim, xlim, ylim, nx, ny = param_set
    N=data.shape[0]
    phase_bin=np.linspace(xlim[0], xlim[1],nx+1)
    axis_bin=np.linspace(ylim[0], ylim[1], ny+1)

    fig = pyplot.figure(dpi=300)
    ax = fig.add_subplot(1, 1, 1)

    ax.set_xlabel(f"$\\epsilon$")
    ax.set_ylabel(f"$n_z$")
    ax.set_title(f"{dim}D, h={h}")
    density, _, _, im = ax.hist2d(data.phase_err, data.z_err, 
    bins=[phase_bin, axis_bin],
    density=False,
    # cmap="magma",
    # cmap="inferno",
    # cmap="plasma",
    # cmap=plt.cm.gray,
    # norm=plt.colors.LogNorm()
    )
    ax.ticklabel_format(axis="both",style="sci", scilimits=(0,0))
    if rotation:
        pyplot.xticks(rotation=45)
    # cbar = fig.colorbar(im, ax=ax)

    cb_ticks = np.linspace(density.min(), density.max(), 5, endpoint=True)
    cbar = fig.colorbar(im, ax=ax, ticks=cb_ticks)
    cbar.set_ticklabels(map(sci_fmt,cb_ticks))
    fig.savefig(out_dir+f"{dim}D_driving_error_h={h}.png", bbox_inches='tight')