"""
Assert a constant driving field h at [1,0,0] direction (x axis) 
find the driving error via Monte-Carlo sampling
"""
function simulatedrivingerror(sim::Simulation)
    @assert sim.system isa SpinEnsemble
    @assert sim.h isa SquarePulse
    pulse=sim.h
    
    t_π=rabiperiod(sim.system, pulse.h, M=sim.M, N=sim.N, λ=0.2, L=sim.nt)

    Δt=t_π*pulse.phi/π
    err=zeros(sim.N*sim.M,4)
    
    @showprogress for k in 1:sim.M
        cluster = SpinCluster(sim.system)
        phi, n = rabisampling(pulse.h, Δt, cluster, pulse.aim, N=sim.N)
        err[(k-1)*sim.N+1:k*sim.N,1] = phi.-pulse.phi 
        err[(k-1)*sim.N+1:k*sim.N, 2:4] = n .- transpose(pulse.aim)
        SpinControl.SpinEnsembles.reroll!(cluster)
    end

    return DataFrame(err, [:phase_err,:x_err, :y_err, :z_err])
end
