import Statistics: mean, std

function simulatesequencetrace(sim::Simulation)
    @assert sim.system isa SpinEnsemble
    @assert sim.protocol isa SquareSequence
    @assert sim.h isa Real
    @assert sim.cache

    dim=sim.system.dim
    nt=sim.nt 
    cycle=sim.cycle 
    seq=sim.protocol
    ρ=sim.ρ
    β = cache[!,Symbol("dist_$(dim)d")][1:sim.M]
    
    t_arr, ρ_arr = deploy(ρ, seq, nt, β, cycle=cycle)
    return t_arr, ρ_arr
end

function simulatesequencefidelity(sim::Simulation)
    @assert sim.system isa SpinEnsemble
    @assert sim.protocol isa SquareSequence
    @assert sim.h isa Real
    @assert sim.cache

    dim=sim.system.dim
    nt=sim.nt 
    cycle=sim.cycle 
    seq=sim.protocol

    for ρ in [σ_x,σ_y,σ_z]
        ρ=sim.ρ
        β = cache[!,Symbol("dist_$(dim)d")][1:sim.M]
        
        t_arr, ρ_arr = deploy(ρ, seq, nt, β, cycle=cycle)

        F_arr=map(x->measure(ρ',x), ρ_arr)
    end
    
    return t_arr, ρ_arr
end

function samplefidelity(sim::Simulation, fidelity::Function)
    @assert sim.system isa SpinEnsemble
    @assert sim.h isa Vector
    @assert sim.cache

    n=sim.system.dim
    β = cache[!,Symbol("dist_$(n)d")][1:sim.M]
    τ=sim.dt
    
    data=zeros(length(β),length(sim.h))
    @showprogress for j in 1:length(sim.h)
        h=sim.h[j]
        ϵ=rabiperiod(β, h)-π/h
        for i in 1:length(β) 
            b=β[i]
            data[i,j] = fidelity(b,h,τ,ϵ)
        end
    end
    data=vcat(mean(data, dims=1),std(data, dims=1)./sqrt(sim.M))

    label=["h=$h" for h in sim.h]
    return DataFrame(data,label)
end

