include("../src/simulations.jl")
import Random: shuffle!

@time cache=readsimulation("c57742e83b8adcc0", "field_dist/", verbose=false)
shuffle!(cache.dist_3d)
shuffle!(cache.dist_2d)
shuffle!(cache.dist_1d)

println("cache loaded!")

sim_list=[]
sub_dir="cache/"
h_list=[20,50,100,200]
τ_list=[1]
M=convert(Int, 4e5)

function preparekrausops(sim::Simulation)
    @assert sim.system isa SpinEnsemble
    @assert sim.cache
    n=sim.system.dim
    β = cache[!,Symbol("dist_$(n)d")][1:sim.M]
    if sim.h isa Union{Sequence,Pulse}
        kops=krausoperators(sim.h, β)
    elseif sim.h isa Vector
        kops=krausoperators(rabisampling(sim.h, β, sim.system.z0)...)
    elseif sim.h isa Real
        kops=krausoperators(rabisampling(sim.h*[1,0,0], β, sim.system.z0)...)
    end
    return kops
end

# 3D
β = cache[!,Symbol("dist_3d")][1:M]

for h in h_list #[1:1]
    ϵ=rabiperiod(β,h)-π/h
    for τ in τ_list
        seq=CP(h,τ,ϵ)
        sim=Simulation(system=ensemble3d,
        h=seq,
        M=M,
        description="Produce krausoperators of CP protocol, 3d, h=$(h), τ=$τ", 
        outdir=DATADIR*sub_dir,
        cache=true,
        execution=preparekrausops)
        push!(sim_list,sim)

        seq=XY(h,τ,ϵ)
        sim=Simulation(system=ensemble3d,
        h=seq,
        M=M,
        description="Produce krausoperators of XY protocol, 3d, h=$(h), τ=$τ", 
        outdir=DATADIR*sub_dir,
        cache=true,
        execution=preparekrausops)
        push!(sim_list,sim)
    end
end

# readsimulation("64f80c97c6dfb7d5","cache/", fmt=".jld")

id_list=map(id, sim_list)
println(id_list)
@assert length(unique(id_list))==8
map(execute, sim_list)