include("../src/simulations.jl")
import Random: shuffle!

@time cache=readsimulation("c57742e83b8adcc0", "field_dist/", verbose=false)
shuffle!(cache.dist_3d)
shuffle!(cache.dist_2d)
shuffle!(cache.dist_1d)

println("cache loaded!")

#### 
# parameters to sweep h, τ, dim


# 3D
sim_list=[]
sub_dir="sequence_trace/APCP_trace/"
M=convert(Int, 2e5)

h=200
h_list=[h]
tp=pi/h
τ_list=[0.4tp, 0.5tp, tp, 1.5tp, 2tp, 5tp ]
# τ_list=[0.1tp, 0.5tp, tp]
n_cycle=400
nt=1
ρ0=[1.0+0im -1.0im; 1.0im 1.0+0im ]/2

for h in h_list
    for τ in τ_list
        β = cache[!,Symbol("dist_3d")][1:M]
        δ=0
        seq=APCP(h,τ,δ)
        sim=Simulation(
            system=ensemble3d,
            protocol=seq,
            ρ=ρ0,
            cycle=n_cycle,
            nt=nt,
            h=h,
            dt=τ,
            M=M,
            description="APCP dynamics trace, 3D, h=$h, τ=$τ", 
            outdir=DATADIR*sub_dir,
            cache=true,
            execution=simulatesequencetrace
        )
        push!(sim_list,sim)
    end
end

# # 2D
# for h in h_list
#     for τ in τ_list
#         β = cache[!,Symbol("dist_2d")][1:M]
#         δ=0
#         seq=APCP(h,τ,δ)
#         sim=Simulation(
#             system=ensemble2d,
#             protocol=seq,
#             ρ=ρ0,
#             cycle=n_cycle,
#             nt=nt,
#             h=h,
#             dt=τ,
#             M=M,
            
#             description="APCP dynamics trace, 2D, h=$h, τ=$τ", 
#             outdir=DATADIR*sub_dir,
#             cache=true,
#             execution=simulatesequencetrace
#         )
#         push!(sim_list,sim)
#     end
# end

# 1D

# for h in h_list
#     for τ in τ_list
#         β = cache[!,Symbol("dist_1d")][1:M]
#         δ=0
#         seq=APCP(h,τ,δ)

#         sim=Simulation(
#         system=ensemble1d,
#         protocol=seq,
#         ρ=ρ0,
#         cycle=n_cycle,
#         nt=nt,
#         h=h,
#         dt=τ,
#         M=M,

#         description="APCP dynamics trace, 1D, h=$h, τ=$τ", 
#         outdir=DATADIR*sub_dir,
#         cache=true,
#         execution=simulatesequencetrace
#         )
#         push!(sim_list,sim)
#     end
# end

id_list=map(id, sim_list)
println("Simulation ids:")
println(id_list)
@assert length(unique(id_list))==length(τ_list)*length(h_list)*1
map(execute, sim_list)