include("../src/simulations.jl")
import Random: shuffle!

@time cache=readsimulation("c57742e83b8adcc0", "field_dist/", verbose=false)
shuffle!(cache.dist_3d)
shuffle!(cache.dist_2d)
shuffle!(cache.dist_1d)

println("cache loaded!")

#### 
# parameters to sweep h, τ, dim


dim=1
sim_list=[]
sub_dir="sequence_trace/WA_trace/"
M=convert(Int, 1e5)

t_lim=4
h_list=[200]
nt=3
ρ0=(σ_i+σ_y)/2


for h in h_list
    tp=pi/h/2
    τ_list=[0.4tp, 0.5tp, tp, 1.5tp, 2tp, 5tp ]
    for τ in τ_list
        tc=8τ+4tp
        n_cycle=convert(Int, floor(t_lim/tc))

        β = cache[!,Symbol("dist_$(dim)d")][1:M]
        δ=rabiperiod(β, h)-π/h
        seq=WAHUHA(h,τ,δ)
        sim=Simulation(
            system=[ensemble1d,ensemble2d,ensemble3d][dim],
            protocol=seq,
            ρ=ρ0,
            cycle=n_cycle,
            nt=nt,
            h=h,
            dt=τ,
            M=M,
            description="WAHUHA dynamics trace, $(dim)D, h=$h, τ=$τ", 
            outdir=DATADIR*sub_dir,
            cache=true,
            execution=simulatesequencetrace
        )
        push!(sim_list,sim)
    end
end

id_list=map(id, sim_list)
println("Simulation ids:")
println(id_list)
@assert length(unique(id_list))==6
map(execute, sim_list)